﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Controls.Schedules;
using Microsoft.Win32;

using WPfIgGantImportExport.Models;

using Infragistics;

using System.IO;
using System.Xml;
using System.Xml.Serialization;


using msproject = Microsoft.Office.Interop.MSProject;
using System.Reflection;

namespace WPfIgGantImportExport
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        #region Constructors
        public MainWindow()
        {
            InitializeComponent();
        }
        #endregion Constructors

        #region Methods

        #region AddNewCustomTask
        private void AddNewCustomTask(ProjectViewModel model)
        {

            TaskModel newTask = new TaskModel();
            newTask.DurationFormat = Infragistics.Controls.Schedules.ProjectDurationFormat.ElapsedDays;
            newTask.Duration = new TimeSpan(8, 0, 0);
            newTask.IsMilestone = false;
            IEnumerable<TaskModel> existingTasks = null;

            do
            {
                int taskNumber = RandomNumber(1, 1000);

                newTask.TaskName = "Task" + taskNumber.ToString();
                existingTasks = model.Tasks.Where(a => a.TaskName.Equals(newTask.TaskName));
            } while (existingTasks.Count() > 0);

            newTask.Start = DateTime.Now;
            newTask.IsUndetermined = true;
            model.SelectedTask = null;

            model.Tasks.Add(newTask);

            model.SelectedTask = newTask;

        }
        #endregion AddNewCustomTask

        #region DeleteCustomTask
        private void DeleteCustomTask(ProjectViewModel model)
        {
            if (model.SelectedTask != null)
            {
                model.Tasks.Remove(model.SelectedTask);
                model.SelectedTask = null;
            }
        }
        #endregion DeleteCustomTask

        #region DeleteNativeTask
        private void DeleteNativeTask(NativeProjectViewModel model)
        {
            if (model.SelectedTask != null && model.SelectedTask != model.Project.RootTask)
            {
                model.Project.RootTask.Tasks.Remove(model.SelectedTask);
                model.SelectedTask = null;
            }
        }
        #endregion DeleteNativeTask

        #region AddNewNativeTask
        private void AddNewNativeTask(NativeProjectViewModel model)
        {

            ProjectTask newTask = new ProjectTask();
            newTask.DurationFormat = Infragistics.Controls.Schedules.ProjectDurationFormat.ElapsedDays;
            newTask.Duration = new TimeSpan(8, 0, 0);
            newTask.IsMilestone = false;
            IEnumerable<ProjectTask> existingTasks = null;

            do
            {
                int taskNumber = RandomNumber(1, 1000);

                newTask.TaskName = "Task" + taskNumber.ToString();
                existingTasks = model.Project.RootTask.Tasks.Where(a => a.TaskName.Equals(newTask.TaskName));
            } while (existingTasks.Count() > 0);


            newTask.Start = DateTime.Now;

            model.SelectedTask = null;

            model.Project.RootTask.Tasks.Add(newTask);

            model.SelectedTask = newTask;

        }
        #endregion AddNewNativeTask


        #region ExportProject
        //Idea how to export project using XmlSerializer and MS Project XML schema
        private void ExportProject(NativeProjectViewModel model)
        {
            WPfIgGantImportExport.Models.Export.Project exportProject = new Models.Export.Project();

            ProjectTask rootTask = model.Project.RootTask;

            WPfIgGantImportExport.Models.Export.ProjectTask exportTask = new Models.Export.ProjectTask();


            exportTask.ActualDuration = rootTask.ActualDuration.ToString();
            if (rootTask.ActualFinish != null)
            {
                exportTask.ActualFinish = (DateTime)rootTask.ActualFinish;
            }

            if (rootTask.ActualStart != null)
            {
                exportTask.ActualStart = (DateTime)rootTask.ActualStart;
            }


            exportTask.Start = (DateTime)rootTask.Start;
            exportTask.Finish = (DateTime)rootTask.Finish;

            if (rootTask.Duration != null)
            {
                exportTask.Duration = rootTask.Duration.ToString();
            }


            if (rootTask.TaskName != null)
            {
                exportTask.Name = rootTask.TaskName;
            }


            exportTask.ID = rootTask.Id.ToString();
            exportTask.UID = rootTask.Id.ToString();

            exportProject.Tasks = new Models.Export.ProjectTask[1];

            exportProject.Tasks[0] = exportTask;


            Microsoft.Win32.SaveFileDialog dlg = new Microsoft.Win32.SaveFileDialog();
            dlg.FileName = "Document"; // Default file name
            dlg.DefaultExt = ".xml"; // Default file extension
            dlg.Filter = "XML documents (.xml)|*.xml"; // Filter files by extension

            Nullable<bool> result = dlg.ShowDialog();

            // Process save file dialog box results
            if (result == true)
            {
                // Save document
                string filename = dlg.FileName;
                XmlSerializer serializer = new XmlSerializer(typeof(WPfIgGantImportExport.Models.Export.Project));
                TextWriter textWriter = new StreamWriter(filename);
                serializer.Serialize(textWriter, exportProject);
                textWriter.Close();
            }

        }
        #endregion ExportProject


        #region GenerateCustomTask
        private void GenerateCustomTask(ProjectViewModel model)
        {
            TaskModel task = new TaskModel();
            task.DurationFormat = Infragistics.Controls.Schedules.ProjectDurationFormat.ElapsedDays;
            task.ConstraintDate = new DateTime(2012, 10, 10, 8, 0, 0);
            task.ConstraintType = ProjectTaskConstraintType.FinishNoLaterThan;
            task.Duration = new TimeSpan(80, 0, 0);

            task.IsMilestone = false;
            task.TaskName = "Sample task";

            IEnumerable<TaskModel> existingTasks = null;

            do
            {
                int taskNumber = RandomNumber(1, 1000);

                task.TaskName = "Sample task" + taskNumber.ToString();
                task.TaskID = task.TaskName;
                existingTasks = model.Tasks.Where(a => a.TaskName.Equals(task.TaskName));
            } while (existingTasks.Count() > 0);

            task.Start = new DateTime(2012, 10, 2);

            task.IsUndetermined = true;
            model.SelectedTask = null;

            model.Tasks.Add(task);
            model.SelectedTask = task;

        }
        #endregion //GenerateCustomTask

        #region GenerateNativeTask
        private void GenerateNativeTask(NativeProjectViewModel model)
        {
            ProjectTask task = new ProjectTask();
            task.DurationFormat = Infragistics.Controls.Schedules.ProjectDurationFormat.ElapsedDays;
            task.ConstraintDate = new DateTime(2012, 10, 10, 8, 0, 0);
            task.ConstraintType = ProjectTaskConstraintType.FinishNoLaterThan;
            task.Duration = new TimeSpan(80, 0, 0);

            task.IsMilestone = false;
            task.TaskName = "Sample task";

            IEnumerable<ProjectTask> existingTasks = null;

            do
            {
                int taskNumber = RandomNumber(1, 1000);

                task.TaskName = "Sample task" + taskNumber.ToString();

                existingTasks = model.Project.RootTask.Tasks.Where(a => a.TaskName.Equals(task.TaskName));
            } while (existingTasks.Count() > 0);

            task.Start = new DateTime(2012, 10, 2);

            model.SelectedTask = null;

            model.Project.RootTask.Tasks.Add(task);
            model.SelectedTask = task;

        }
        #endregion GenerateNativeTask


        #region RandomNumber
        private int RandomNumber(int min, int max)
        {
            Random random = new Random();
            return random.Next(min, max);
        }
        #endregion //RandomNumber

        #region ReadGanttProject
        private void ReadGanttProject(ref ProjectViewModel model)
        {
            Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();
            dlg.FileName = "Document"; // Default file name
            dlg.DefaultExt = ".xml"; // Default file extension
            dlg.Filter = "XML documents (.xml)|*.xml"; // Filter files by extension

            Nullable<bool> result = dlg.ShowDialog();

            // Process save file dialog box results
            if (result == true)
            {
                // Save document
                string filename = dlg.FileName;


                XmlSerializer deserializer = new XmlSerializer(typeof(ProjectViewModel));
                TextReader textReader = new StreamReader(filename);
                var newModel = (ProjectViewModel)deserializer.Deserialize(textReader);

                model.Tasks.Clear();

                foreach (TaskModel task in newModel.Tasks)
                {

                    model.Tasks.Add(task);
                    //Update PredecessorsIdText
                    xamGantt.Project.RootTask.Tasks.Last().PredecessorsIdText = task.PredecessorsIdText;
                }

                textReader.Close();
            }
        }
        #endregion ReadGanttProject

        #region SaveGanttProject
        private void SaveGanttProject(ProjectViewModel model)
        {
            Microsoft.Win32.SaveFileDialog dlg = new Microsoft.Win32.SaveFileDialog();
            dlg.FileName = "Document"; // Default file name
            dlg.DefaultExt = ".xml"; // Default file extension
            dlg.Filter = "XML documents (.xml)|*.xml"; // Filter files by extension

            Nullable<bool> result = dlg.ShowDialog();

            // Process save file dialog box results
            if (result == true)
            {
                // Save document
                string filename = dlg.FileName;
                XmlSerializer serializer = new XmlSerializer(typeof(ProjectViewModel));
                TextWriter textWriter = new StreamWriter(filename);
                serializer.Serialize(textWriter, model);
                textWriter.Close();
            }
        }
        #endregion SaveGanttProject

        #region UpdatePredecessorsIdText
        private void UpdatePredecessorsIdText(XamGantt gantt)
        {
            foreach (ProjectTask task in gantt.Project.RootTask.Tasks)
            {
                var customTask = task.DataItem as TaskModel;
                customTask.PredecessorsIdText = task.PredecessorsIdText;
            }
        }
        #endregion UpdatePredecessorsIdText

        #endregion Methods


        #region EventHandlers

        #region btnDeleteTask_Click
        private void btnDeleteTask_Click(object sender, RoutedEventArgs e)
        {
            var tabIndex = this.ganttTabControl.SelectedIndex;

            if (tabIndex == 0)
            {
                var model = this.Root.Resources["viewmodel"] as ProjectViewModel;
                if (model != null)
                {
                    DeleteCustomTask(model);

                }
            }

            else if (tabIndex == 1)
            {
                var model = this.Root.Resources["nativeviewmodel"] as NativeProjectViewModel;
                if (model != null)
                {
                    DeleteNativeTask(model);

                }
            }
        }
        #endregion btnDeleteTask_Click



        #region btnNewTask_Click
        private void btnNewTask_Click(object sender, RoutedEventArgs e)
        {
            var tabIndex = this.ganttTabControl.SelectedIndex;

            if (tabIndex == 0)
            {
                var model = this.Root.Resources["viewmodel"] as ProjectViewModel;
                if (model != null)
                {
                    AddNewCustomTask(model);

                }
            }

            else if (tabIndex == 1)
            {
                var model = this.Root.Resources["nativeviewmodel"] as NativeProjectViewModel;
                if (model != null)
                {
                    AddNewNativeTask(model);

                }
            }

        }


        #endregion //btnNewTask_Click

        #region btnGenerateTask_Click
        private void btnGenerateTask_Click(object sender, RoutedEventArgs e)
        {
            //check for selected tab

            var tabIndex = this.ganttTabControl.SelectedIndex;

            if (tabIndex == 0)
            {
                var model = this.Root.Resources["viewmodel"] as ProjectViewModel;
                if (model != null)
                {
                    GenerateCustomTask(model);

                    ProjectTask firstTask = this.xamGantt.Project.RootTask.Tasks[0];

                    ProjectTask lastTask = this.xamGantt.Project.RootTask.Tasks.Last<ProjectTask>();
                                        
                    //lastTask.PredecessorsIdText

                    if (!firstTask.Equals(lastTask))
                    {
                        lastTask.Predecessors.Add(firstTask, ProjectTaskLinkType.FinishToStart, ProjectDuration.FromFormatUnits(-2, ProjectDurationFormat.Days));

                    }

                }
            }

            else if (tabIndex == 1)
            {
                var model = this.Root.Resources["nativeviewmodel"] as NativeProjectViewModel;
                if (model != null)
                {
                    GenerateNativeTask(model);

                    ProjectTask firstTask = this.nativeGantt.Project.RootTask.Tasks[0];

                    ProjectTask lastTask = this.nativeGantt.Project.RootTask.Tasks.Last<ProjectTask>();

                    if (!firstTask.Equals(lastTask))
                    {
                        lastTask.Predecessors.Add(firstTask, ProjectTaskLinkType.FinishToStart, ProjectDuration.FromFormatUnits(-2, ProjectDurationFormat.Days));

                    }

                }
            }


        }


        #endregion //btnGenerateTask_Click

        #region btnRead_Click
        private void btnRead_Click(object sender, RoutedEventArgs e)
        {
            var tabIndex = this.ganttTabControl.SelectedIndex;

            if (tabIndex == 0)
            {
                var model = this.Root.Resources["viewmodel"] as ProjectViewModel;
                if (model != null)
                {
                    ReadGanttProject(ref model);
                }
            }

            else if (tabIndex == 1)
            {
                var model = this.Root.Resources["nativeviewmodel"] as NativeProjectViewModel;
                if (model != null)
                {

                    MessageBox.Show("XamGantt Project doesn't support XML serialization");

                }
            }
        }
        #endregion btnRead_Click

        #region btnSave_Click
        private void btnSave_Click(object sender, RoutedEventArgs e)
        {
            var tabIndex = this.ganttTabControl.SelectedIndex;

            if (tabIndex == 0)
            {
                var model = this.Root.Resources["viewmodel"] as ProjectViewModel;
                if (model != null)
                {
                    UpdatePredecessorsIdText(xamGantt);
                    SaveGanttProject(model);
                }
            }

            else if (tabIndex == 1)
            {
                var model = this.Root.Resources["nativeviewmodel"] as NativeProjectViewModel;
                if (model != null)
                {

                    MessageBox.Show("XamGantt Project doesn't support XML serialization");
                }
            }
        }
        #endregion btnSave_Click

        #region GanttTabControl_SelectionChanged
        private void GanttTabControl_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            TabItem selectedTab = e.AddedItems[0] as TabItem;
            if (selectedTab.Name == "CustomTaskTabItem")
            {
                var model = this.Root.Resources["viewmodel"] as ProjectViewModel;
                this.selectedTaskPanel.DataContext = model;
            }
            else if (selectedTab.Name == "BindToProjectTabItem")
            {
                var model = this.Root.Resources["nativeviewmodel"] as NativeProjectViewModel;
                this.selectedTaskPanel.DataContext = model;
            }

        }
        #endregion GanttTabControl_SelectionChanged

        #region nativeGantt_ActiveCellChanged
        private void nativeGantt_ActiveCellChanged(object sender, EventArgs e)
        {
            if (this.nativeGantt.ActiveRow.HasValue)
            {

                var task = ((Infragistics.Controls.Schedules.ProjectTask)(this.nativeGantt.ActiveRow.Value.Task)) as ProjectTask;

                if (task != null)
                {
                    var model = this.Root.Resources["nativeviewmodel"] as NativeProjectViewModel;
                    model.SelectedTask = task;
                }

            }
        }
        #endregion //nativeGantt_ActiveCellChanged

        #region xamGantt_ActiveCellChanged
        private void xamGantt_ActiveCellChanged(object sender, EventArgs e)
        {

            if (this.xamGantt.ActiveRow.HasValue)
            {

                var task = ((Infragistics.Controls.Schedules.ProjectTask)(this.xamGantt.ActiveRow.Value.Task)).DataItem as TaskModel;

                if (task != null)
                {
                    var model = this.Root.Resources["viewmodel"] as ProjectViewModel;
                    model.SelectedTask = task;
                }

            }
        }
        #endregion //xamGantt_ActiveCellChanged

        #endregion EventHandlers

    }
}
